@@ -9,7 +9,7 @@ use IO::Socket::INET;
 use MogileFS::Server;
 use base 'Exporter';
 
-our @EXPORT = qw(&find_mogclient_or_skip &temp_store &create_mogstored &create_temp_tracker &try_for);
+our @EXPORT = qw(&find_mogclient_or_skip &temp_store &create_mogstored &create_temp_tracker &try_for &want);
 
 sub find_mogclient_or_skip {
 
@@ -161,6 +161,34 @@ sub try_for {
     return 0;
 }
 
+sub want {
+    my ($admin, $count, $jobclass) = @_;
+    my $req = "!want $count $jobclass\r\n";
+
+    syswrite($admin, $req) or die "syswrite: $!\n";
+
+    my $r = <$admin>;
+    if ($r =~ /Now desiring $count children doing '$jobclass'/ && <$admin> eq ".\r\n") {
+	    my $rcount;
+	    try_for(30, sub {
+            $rcount = -1;
+            syswrite($admin, "!jobs\r\n");
+            MogileFS::Util::wait_for_readability(fileno($admin), 10);
+            while (1) {
+                my $line = <$admin>;
+                if ($line =~ /\A$jobclass count (\d+)/) {
+                    $rcount = $1;
+                }
+                last if $line eq ".\r\n";
+            }
+            $rcount == $count;
+        });
+        return 1 if $rcount == $count;
+        die "got $jobclass count $rcount (expected=$count)\n";
+    }
+    die "got bad response for $req: $r\n";
+}
+
 ############################################################################
 package ProcessHandle;
 sub new {
